"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GitlabConnector = void 0;
const node_fetch_1 = require("node-fetch");
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
const unzipper_1 = require("unzipper");
const colors = require("colors");
const connector_1 = require("./connector");
class GitlabConnector extends connector_1.PluginConnector {
    getReleases() {
        return (0, node_fetch_1.default)(`${this.url}/repository/tags`, {
            headers: this.buildHeader(this.token)
        })
            .then(res => res.json())
            .then((res) => {
            return res.map((r) => {
                return {
                    versionName: r.name,
                    commitId: r.target,
                    config: JSON.parse(r.release.description)
                };
            });
        })
            .catch(e => {
            if (e instanceof node_fetch_1.FetchError && e.code === 'ENOTFOUND') {
                console.error(`${colors.red('Repo not accessible.')} ${colors.cyan('Check your VPN ;)')}`);
            }
            else {
                throw e;
            }
            return [];
        });
    }
    async download(release, tempPath) {
        const res = await (0, node_fetch_1.default)(`${this.url}/repository/archive.zip?sha=${release.commitId}`, {
            headers: this.buildHeader(this.token)
        });
        const tmpPath = (0, path_1.join)(tempPath, 'tmp');
        await new Promise((resolve, reject) => {
            if (res && res.body) {
                res.body.pipe((0, unzipper_1.Extract)({ path: tmpPath }));
                res.body.on('error', reject);
                res.body.on('end', resolve);
            }
            else {
                reject('No body on response.');
            }
        });
        const elements = (0, fs_extra_1.readdirSync)(tmpPath, { withFileTypes: true });
        const tempExtractName = elements[0].name;
        (0, fs_extra_1.copySync)((0, path_1.join)(tmpPath, tempExtractName), `${tempPath}`);
        (0, fs_extra_1.removeSync)(tmpPath);
    }
    buildHeader(token) {
        if (token) {
            return {
                'PRIVATE-TOKEN': token
            };
        }
        return {};
    }
}
exports.GitlabConnector = GitlabConnector;
